DATA_VERSION = '0'
RECORD_OFFSET_SIZE = 4
RECORD_LEN_SIZE = 4
BUCKET_OFFSET_SIZE = 8
RECORDS_NUM_SIZE = 8
BUCKET_SIZE_KB = 1024
ENDIANNESS = 'little'

RECORDS_NUM = 'records_num'
BUCKETS_NUM = 'buckets_num'
COMPRESSION = 'compression'

COMPRESSION_GZIP = 'gzip'
COMPRESSION_ALGORITHMS = (
    None,
    COMPRESSION_GZIP,
)
